/* Teensyduino Core Library
 * http://www.pjrc.com/teensy/
 * Copyright (c) 2013 PJRC.COM, LLC.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * 1. The above copyright notice and this permission notice shall be 
 * included in all copies or substantial portions of the Software.
 *
 * 2. If the Software is incorporated into a build system that allows 
 * selection among a list of target devices, then similar target
 * devices manufactured by PJRC.COM must be included in the list of
 * target devices and selectable in the same manner.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef USBjoystick_h_
#define USBjoystick_h_

#if defined(USB_HID) || defined(USB_SERIAL_HID)

#include <inttypes.h>

// C language implementation
#ifdef __cplusplus
extern "C" {
#endif
int usb_joystick_send(void);
extern uint32_t usb_joystick_data[16];
#ifdef __cplusplus
}
#endif

// C++ interface
#ifdef __cplusplus
class usb_joystick_class
{
        public:
        void begin(void) { }
        void end(void) { }
	void button(unsigned int num, bool val) {
		if (--num >= 128) return;
		uint32_t *p = usb_joystick_data + (num >> 5);
		num &= 0x1F;
		if (val) *p |= (1 << num);
		else *p &= ~(1 << num);
		if (!manual_mode) usb_joystick_send();
	}
	void X(unsigned int position) { analog16(0, position); }
	void Y(unsigned int position) { analog16(1, position); }
	void Z(unsigned int position) { analog16(2, position); }
	void Xrotate(unsigned int position) { analog16(3, position); }
	void Yrotate(unsigned int position) { analog16(4, position); }
	void Zrotate(unsigned int position) { analog16(5, position); }
	void slider(unsigned int num, unsigned int position) {
		if (--num >= 15) return;
		analog16(num + 6, position);
	}
        inline void hat(unsigned int num, int angle) {
                if (angle < 0 || angle > 359) angle = 0xFFF;
		uint32_t *p = usb_joystick_data;
		switch(num) {
		  case 1:
			p[14] = (p[14] & 0xF000FFFF) | (angle << 16);  break;
		  case 2:
			p[14] = (p[14] & 0x0FFFFFFF) | (angle << 28);
			p[15] = (p[15] & 0xFFFFFF00) | (angle >> 4);   break;
		  case 3:
			p[15] = (p[15] & 0xFFF000FF) | (angle << 8);   break;
		  case 4:
			p[15] = (p[15] & 0x000FFFFF) | (angle << 20);  break;
		  default:
			return;
		}
		if (!manual_mode) usb_joystick_send();
        }
	void useManualSend(bool mode) {
		manual_mode = mode;
	}
	void send_now(void) {
		usb_joystick_send();
	}
	private:
	void analog16(unsigned int num, unsigned int value) {
		if (value > 0xFFFF) value = 0xFFFF;
		uint16_t *p = (uint16_t *)(&usb_joystick_data[4]);
		p[num] = value;
                if (!manual_mode) usb_joystick_send();
	}
	static uint8_t manual_mode;
};
extern usb_joystick_class Joystick;

#endif // __cplusplus

#endif // USB_HID || USB_SERIAL_HID
#endif // USBjoystick_h_

